#include <RCS620S.h>
#include "oled_functions.h"

#define COMMAND_TIMEOUT               400
#define POLLING_INTERVAL              500
#define RCS620S_MAX_CARD_RESPONSE_LEN 30

// FeliCa Service/System Code
#define CYBERNE_SYSTEM_CODE           0x0003
#define COMMON_SYSTEM_CODE            0xFE00
#define PASSNET_SERVICE_CODE          0x090F
#define EDY_SERVICE_CODE              0x170F
#define NANACO_SERVICE_CODE           0x564F
#define WAON_SERVICE_CODE             0x680B

RCS620S rcs620s;

// UNO
#define SERIAL_DEBUG Serial
// Due
// #define SERIAL_DEBUG SerialUSB

void setup() {
  int ret;
  while (!Serial && millis() < 3000);
  Serial.begin(115200);
  Serial.println("Serial init done!");

  setup_oled();

  display.setTextColor(SH110X_WHITE);
  display.setTextSize(2);
  display.setCursor(OLED_COL0,OLED_ROW0);
  display.print("FeliCa");
  display.display();

  // RC-S620Sの初期化
  Serial1.begin(115200);
  Serial.println("Serial1 init done!");
  ret = rcs620s.initDevice();
  while (!ret) {}
  rcs620s.timeout = COMMAND_TIMEOUT;
  Serial.println("RC-S620S init ok");
  display.setCursor(0, OLED_ROW2);
  display.setTextSize(1);
  display.print("RC-S620S init ok");
  display.display();
}

void polling(void) {
  int ret;
  char buf[30];

  // FeliCaのタッチ状態を得る
  ret = rcs620s.polling();

  // FeliCaがタッチされた場合
  if(ret) {
    // IDmを取得する
    display.fillRect(OLED_COL0, OLED_ROW3, 128, OLED_ROW3 + 8, SH110X_BLACK);
    display.setCursor(0, OLED_ROW3);
    display.setTextSize(1);
    display.print("IDm=");
    for(int i = 0; i < 8; i++){
      sprintf(buf, "%02X", rcs620s.idm[i]);
      Serial.print(buf);
      display.print(buf);
    }
    Serial.println();
    display.display();
  }
  
  rcs620s.rfOff();
}

void polling_balance(void) {
  char temp[3];
  char cardid[17];
  uint32_t balance;
  uint8_t buf[RCS620S_MAX_CARD_RESPONSE_LEN];
   
  rcs620s.timeout = COMMAND_TIMEOUT;
   
  // サイバネ領域
  if(rcs620s.polling(CYBERNE_SYSTEM_CODE)){
    // Suica PASMO
    if(requestService(PASSNET_SERVICE_CODE)){
      if(readEncryption(PASSNET_SERVICE_CODE, 0, buf)){
        // Little Endianで入っているPASSNETの残高を取り出す
        balance = buf[23];                  // 11 byte目
        balance = (balance << 8) + buf[22]; // 10 byte目

        // IDmを取得する
        for(int i = 0; i < 8; i++){
          sprintf(temp, "%02X", rcs620s.idm[i]);
          cardid[i * 2] = temp[0];
          cardid[(i * 2) + 1] = temp[1];
        }
        cardid[16] = 0;

        // 残高表示
        printBalanceLCD(cardid, "PASSNET", &balance);
      }
    }
  }
   
  // 共通領域
  else if(rcs620s.polling(COMMON_SYSTEM_CODE)){
    // Edy
    if(requestService(EDY_SERVICE_CODE)){
      if(readEncryption(EDY_SERVICE_CODE, 0, buf)){
        // Big Endianで入っているEdyの残高を取り出す
        balance = buf[26];                  // 14 byte目
        balance = (balance << 8) + buf[27]; // 15 byte目

        // IDmを取得する
        for(int i = 0; i < 8; i++){
          sprintf(temp, "%02X", rcs620s.idm[i]);
          cardid[i * 2] = temp[0];
          cardid[(i * 2) + 1] = temp[1];
        }
        cardid[16] = 0;

        // 残高表示
        printBalanceLCD(cardid, "Edy", &balance);
      }
    }
     
    // nanaco
    else if(requestService(NANACO_SERVICE_CODE)){
      if(readEncryption(NANACO_SERVICE_CODE, 0, buf)){
        // Big Endianで入っているNanacoの残高を取り出す
        balance = buf[17];                  // 5 byte目
        balance = (balance << 8) + buf[18]; // 6 byte目
        balance = (balance << 8) + buf[19]; // 7 byte目
        balance = (balance << 8) + buf[20]; // 8 byte目

        // IDmを取得する
        for(int i = 0; i < 8; i++){
          sprintf(temp, "%02X", rcs620s.idm[i]);
          cardid[i * 2] = temp[0];
          cardid[(i * 2) + 1] = temp[1];
        }
        cardid[16] = 0;
        
        // 残高表示
        printBalanceLCD(cardid, "nanaco", &balance);
      }
    }
     
    // waon
    else if(requestService(WAON_SERVICE_CODE)){
      if(readEncryption(WAON_SERVICE_CODE, 1, buf)){
        // Big Endianで入っているWaonの残高を取り出す
        balance = buf[17];                  // 21 byte目
        balance = (balance << 8) + buf[18]; // 22 byte目
        balance = (balance << 8) + buf[19]; // 23 byte目
        balance = balance & 0x7FFFE0;       // 残高18bit分のみ論理積で取り出す
        balance = balance >> 5;             // 5bit分ビットシフト

        // IDmを取得する
        for(int i = 0; i < 8; i++){
          sprintf(temp, "%02X", rcs620s.idm[i]);
          cardid[i * 2] = temp[0];
          cardid[(i * 2) + 1] = temp[1];
        }
        cardid[16] = 0;
        
        // 残高表示
        printBalanceLCD(cardid, "waon", &balance);
      }
    }
  }
   
  rcs620s.rfOff();
}

// request service
int requestService(uint16_t serviceCode){
  int ret;
  uint8_t buf[RCS620S_MAX_CARD_RESPONSE_LEN];
  uint8_t responseLen = 0;
   
  buf[0] = 0x02;
  memcpy(buf + 1, rcs620s.idm, 8);
  buf[9] = 0x01;
  buf[10] = (uint8_t)((serviceCode >> 0) & 0xff);
  buf[11] = (uint8_t)((serviceCode >> 8) & 0xff);
 
  ret = rcs620s.cardCommand(buf, 12, buf, &responseLen);
   
  if(!ret || (responseLen != 12) || (buf[0] != 0x03) ||
      (memcmp(buf + 1, rcs620s.idm, 8) != 0) || ((buf[10] == 0xff) && (buf[11] == 0xff))) {
    return 0;
  }
 
  return 1;
}
 
int readEncryption(uint16_t serviceCode, uint8_t blockNumber, uint8_t *buf){
  int ret;
  uint8_t responseLen = 0;
   
  buf[0] = 0x06;
  memcpy(buf + 1, rcs620s.idm, 8);
  buf[9] = 0x01; // サービス数
  buf[10] = (uint8_t)((serviceCode >> 0) & 0xff);
  buf[11] = (uint8_t)((serviceCode >> 8) & 0xff);
  buf[12] = 0x01; // ブロック数
  buf[13] = 0x80;
  buf[14] = blockNumber;
 
  ret = rcs620s.cardCommand(buf, 15, buf, &responseLen);
 
  if (!ret || (responseLen != 28) || (buf[0] != 0x07) ||
      (memcmp(buf + 1, rcs620s.idm, 8) != 0)) {
    return 0;
  }
 
  return 1;
}

void printIdLCD(char *card_id, char *service_name){
  display.fillRect(OLED_COL0, OLED_ROW3, 128, OLED_ROW3 + 8, SH110X_BLACK);
  display.setCursor(0, OLED_ROW3);
  display.setTextSize(1);
  display.print("IDm=");
  display.print(card_id);
  
  display.fillRect(OLED_COL0, OLED_ROW5, 128, OLED_ROW5 + 8, SH110X_BLACK);
  display.setCursor(0, OLED_ROW5);
  display.setTextSize(1);
  display.print(service_name);
  display.fillRect(OLED_COL0, OLED_ROW6, 128, OLED_ROW6 + 16, SH110X_BLACK);
  display.display();
  
  return;
}

void printBalanceLCD(char *card_id, char *service_name, uint32_t *balance){
  char result[8];

  display.fillRect(OLED_COL0, OLED_ROW3, 128, OLED_ROW3 + 8, SH110X_BLACK);
  display.setCursor(0, OLED_ROW3);
  display.setTextSize(1);
  display.print("IDm=");
  display.print(card_id);
  
  sprintf(result, "%u", *balance);
  display.fillRect(OLED_COL0, OLED_ROW5, 128, OLED_ROW5 + 8, SH110X_BLACK);
  display.setCursor(0, OLED_ROW5);
  display.setTextSize(1);
  display.print(service_name);
  display.fillRect(OLED_COL0, OLED_ROW6, 128, OLED_ROW6 + 16, SH110X_BLACK);
  display.setCursor(0, OLED_ROW6);
  display.setTextSize(2);
  display.print(result);
  display.display();
  
  return;
}

void loop() {
  // put your main code here, to run repeatedly:

  // check for completed serial input
  checkSerialReceiveComplete();

  // check for button press
  checkButtons();

  //polling();
  polling_balance();

  delay(POLLING_INTERVAL);
}

void eventButtonAPressed(void) {
  //display.fillRect(OLED_COL0, OLED_ROW2, 128, 16, SSD1306_BLACK);
  display.fillRect(OLED_COL0, OLED_ROW2, 128, 32, SH110X_BLACK);
  //display.setTextColor(SSD1306_WHITE);
  display.setTextColor(SH110X_WHITE);
  display.setTextSize(1);
  display.setCursor(OLED_COL0,OLED_ROW2);
  display.print("Button A pressed!");
  display.display();
}

void eventButtonBPressed(void) {
  //display.fillRect(OLED_COL0, OLED_ROW2, 128, 16, SSD1306_BLACK);
  display.fillRect(OLED_COL0, OLED_ROW2, 128, 32, SH110X_BLACK);
  //display.setTextColor(SSD1306_WHITE);
  display.setTextColor(SH110X_WHITE);
  display.setTextSize(1);
  display.setCursor(OLED_COL0,OLED_ROW2);
  display.print("Button B pressed!");
  display.display();
}

void eventButtonCPressed(void) {
  //display.fillRect(OLED_COL0, OLED_ROW2, 128, 16, SSD1306_BLACK);
  display.fillRect(OLED_COL0, OLED_ROW2, 128, 32, SH110X_BLACK);
  //display.setTextColor(SSD1306_WHITE);
  display.setTextColor(SH110X_WHITE);
  display.setTextSize(1);
  display.setCursor(OLED_COL0,OLED_ROW2);
  display.print("Button C pressed!");
  display.display();
}
