#include "oled_functions.h"

void setup_oled(void) {
  // put your setup code here, to run once:
  // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  display.begin(0x3C, true); // Address 0x3C default

  // Show image buffer on the display hardware.
  // Since the buffer is intialized with an Adafruit splashscreen
  // internally, this will display the splashscreen.
  //display.display();
  delay(1000);

  // Clear the buffer.
  display.clearDisplay();
  display.setRotation(1);
  display.display();

  buttonInit();

  // check display limits
  display.clearDisplay();
  //display.fillRect(0, 0, 128, 32, SSD1306_WHITE);
  //display.fillRect(1, 1, 126, 30, SSD1306_BLACK);
  display.display();
}

void buttonInit(void) {
  buttonPin[BUTTONA] = BUTTON_A_PIN;
  buttonPin[BUTTONB] = BUTTON_B_PIN;
  buttonPin[BUTTONC] = BUTTON_C_PIN;
  for(uint8_t i = 0; i < BUTTON_COUNT; i++) {
    buttonState[i] = HIGH;
    buttonChangedDelay[i] = 0;
    buttonChangedFlg[i] = false;
    buttonChangedState[i] = HIGH;
    buttonRepeatBeginTime[i] = 0;
    buttonRepeatRepeatTime[i] = 0;
  }
  
  pinMode(BUTTON_A_PIN, INPUT_PULLUP);
  pinMode(BUTTON_B_PIN, INPUT_PULLUP);
  pinMode(BUTTON_C_PIN, INPUT_PULLUP);
}

void buttonChecker(void) {
  bool selectedButtonState;

  for(uint8_t i = 0; i < BUTTON_COUNT; i++) {
    // buttonA
    selectedButtonState = digitalRead(buttonPin[i]);
    if(selectedButtonState == buttonState[i]) {
      // button is same as previous state, are we waiting for a debounce?
      if(buttonChangedDelay[i] != 0) {
        // we are waiting for a debounce
        // has the debounce time expired?
        if((millis() - buttonChangedDelay[i]) > BUTTON_DEBOUNCE_DELAY) {
          // debounce time has expired
          // clear debounce timer
          buttonChangedDelay[i] = 0;
          // set button changed flag
          buttonChangedState[i] = buttonState[i];
          buttonChangedFlg[i] = true;
          if(buttonChangedState[i] == LOW) {
            buttonRepeatBeginTime[i] = millis();
            buttonRepeatRepeatTime[i] = 0;
          } else {
            buttonRepeatBeginTime[i] = 0;
            buttonRepeatRepeatTime[i] = 0;
          }
        }
      }
    } else {
      // button changed state
      // set the new state
      buttonState[i] = selectedButtonState;
      buttonChangedDelay[i] = millis();
      if(buttonChangedDelay[i] == 0) {
        buttonChangedDelay[i] = 1;
      }
    }
  
    // buttonA repeat
    if(buttonChangedState[i] == LOW) {
      // are we in repeat?
      if(buttonRepeatRepeatTime[i] != 0) {
        // we are in repeat
        // check if we can repeat again yet
        if((millis() - buttonRepeatRepeatTime[i]) > BUTTON_REPEAT_REPEAT_DELAY) {
          // set next repeat delay
          buttonRepeatRepeatTime[i] = millis();
          if(buttonRepeatRepeatTime[i] == 0) {
            buttonRepeatRepeatTime[i] = 1;
          }
          buttonChangedState[i] = buttonState[i];
          buttonChangedFlg[i] = true;
        }
      } else if(buttonRepeatBeginTime[i] != 0) {
        // we are not in repeat yet
        // are we delayed enough to start repeating?
        if((millis() - buttonRepeatBeginTime[i]) > BUTTON_REPEAT_BEGIN_DELAY) {
          // zero out the wait delay
          buttonRepeatBeginTime[i] = 0;
          // set next repeat delay
          buttonRepeatRepeatTime[i] = millis();
          if(buttonRepeatRepeatTime[i] == 0) {
            buttonRepeatRepeatTime[i] = 1;
          }
          buttonChangedState[i] = buttonState[i];
          buttonChangedFlg[i] = true;
        }
      }
    }
  }
}

void checkButtons(void) {
  buttonChecker();
  if(buttonChangedFlg[BUTTONA] == true) {
    buttonChangedFlg[BUTTONA] = false;
    if(buttonChangedState[BUTTONA] == LOW) {
      eventButtonAPressed();
    }
  }
  if(buttonChangedFlg[BUTTONB] == true) {
    buttonChangedFlg[BUTTONB] = false;
    if(buttonChangedState[BUTTONB] == LOW) {
      eventButtonBPressed();
    }
  }
  if(buttonChangedFlg[BUTTONC] == true) {
    buttonChangedFlg[BUTTONC] = false;
    if(buttonChangedState[BUTTONC] == LOW) {
      eventButtonCPressed();
    }
  }  
}
